package geometrie;
/**
 * Droite est la classe qui implément les droites de la géométrie.
 *
 * Une droite est une figure géométrique qui est un ensemble de points 
 * alignées.
 * 
 * Les droites sont généralement définies par :
 * <ul>
 * <li>Deux poinns,</li>
 * <li>Un point et un vecteur directeur.</li>
 * </ul>
 *
 */
public class Droite implements FigureGeometrique {
	Point position;
	Vecteur direction;

	/**
	 * Constructeur de la classe à partir d'un point et d'un vecteur directeur.
	 *
	 * @param position Un point appartenant à la droite.
	 * @param direction le vecteur directeur de la droite.
	 */
	public Droite( Point position, Vecteur direction ){
		this.position = new Point(position);
		this.direction = new Vecteur(direction);
	}

	/**
	 * Constructeur de la classe à partir d'un segment appartenant à la droite.
	 *
	 * @param segment Un segment qui appartient à la droite.
	 */
	public Droite( Segment segment ){
		this.position = segment.getPoint1();
		this.direction = segment.getVecteur();
	}

	/**
	 * Constructeur de la classe à partir de deux points du plan.
	 *
	 * Les deux points passés en paramettre doivent impérativement être distinct. 
	 *
	 * @param p1 Premier point appartenant à la droite.
	 * @param p2 Deuxième point appartenant à la droite. Le deuxième point doit
	 *           êtree distinct du premier.
	 */
	public Droite( Point p1, Point p2 ){
		this( new Segment(p1, p2) );
	}

	/**
	 * Constructeur par recopie
	 *
	 * @param droite La droite à recopier.
	 */
	public Droite( Droite droite ){
		this.position = new Point( droite.position );
		this.direction = new Vecteur( droite.direction );
	}

	/**
	 * Renvoie un point par lequel la droite passe.
	 *
	 * @return Un point par lequel la droite passe.
	 */
	Point getPosition(){
		return new Point(this.position);
	}

	/**
	 * Renvoie le vecteur directeur de la droite.
	 *
	 * @return Le vecteur directeur de la droite.
	 */
	Vecteur getDirection(){
		return new Vecteur(this.direction);
	}

	/**
	 * Vérifie si un point donné appartient à la droite.
	 *
	 * @param point Le point qui fera l'objet du test.
	 */
	public boolean isInside( Point point ){
		return ( this.direction.produitVectoriel( new Vecteur( position, point ) ) == 0.0 );
	}
}
