package geometrie;
/**
 * Point est la classe qui implémente les points d'un plan. Il s'agit de
 * mathématiques des espaces affines.
 *
 * Elle fournit de nombreuses méthodes pour effectuer du opération sur les
 * points d'un plan.
 * On y trouve notament les changemenbts de repère.
 */
public class Point extends Couple {
	/**
	 * Constructeur de la classe Point à partir de l'absisse et l'ordonnée du point.
	 *
	 * @param x Absisse du point ( position horizontale )
	 * @param y Ordonnée du point ( position verticale )
	 */
	public Point(double x, double y){
		super(x,y);
	}

	/**
	 * Constructeur par recopie.
	 *
	 * @param p Point à recopier
	 */
	public Point( Point p ){
		super(p);
	}

	/**
	 * Constructeur de la classe Point à partir d'un couple de réel représentant l'absisse et l'ordonnée du point.
	 *
	 * @param c Couple (x,y) représentant l'absisse x et l'ordonne y du point.
	 */
	public Point( Couple c ){
		super(c);
	}

	/**
	 * Translate l'objet à l'aide d'un vecteur.
	 *
	 * @param vecteur Le vecteur de la translation.
	 */
	public void translater( Vecteur vecteur ){
		this.rPlus( vecteur );
	}

	/**
	 * Effectue une rotation de l'objet autour d'une centre et l'aide d'un 
	 * angle orienté dans le sens direct.
	 *
	 * @param centre Le centre de la rotation.
	 * @param rotation L'angle de la rotation.
	 */
	public void effectuerRotation( Point centre, double rotation){
		Vecteur vecteur = this.moins( centre );
		vecteur.effectuerRotation(rotation);
		this.x = centre.x;
		this.y = centre.y;
		this.translater( vecteur );
	}

	/**
	 * Renvoie le vecteur d'origine le vecteur passé en parametre et de fin l'objet.
	 *
	 * Cette méthode implémente la soustraction dans les espaces affines. 
	 *
	 * @param point Le point origine du vecteur.
	 */
	public Vecteur moins( Point point ){
		return new Vecteur( this.x - point.x, this.y - point.y );
	}

	/**
	 * Modifie l'objet de façon à ce que ces coordonnées soit ceux du même
	 * point mais dans un nouveau repère.
	 *
	 * @param positionCentre La position du nouveau repère.
	 * @param rotation L'orientation du nouveau repère.
	 */
	public void changerDeRepere( double rotation, Point positionCentre ){
		this.rMoins(positionCentre);
		this.effectuerRotation( new Point(0.0,0.0), -rotation );
	}

	/**
	 * Rnvoie un nouveau point identique à l'objet dont il est issue, mais
	 * exprimé dans un repère différent.
	 *
	 * @param positionCentre La position du nouveau repère.
	 * @param rotation L'orientation du nouveau repère.
	 */
	public Point changementDeRepere( double rotation, Point positionCentre ){
		Point point = new Point( this );
		point.changerDeRepere( rotation, positionCentre );
		return point;
	}

	/**
	 * Modifie les Coordonnées du point de façon à ce que les nouvelles
	 * coordonnée soient les coordonées où l'ancien point a été place placé
	 * dans le repère passé en paramètre.
	 *
	 * @param positionCentre La position du repère.
	 * @param rotation L'orientation du repère.
	 */
	public void positionnerDansLeRepere( double rotation, Point positionCentre ){
		this.rPlus(positionCentre);
		this.effectuerRotation( positionCentre, rotation );
	}

	/**
	 * Renvoie un nouveau point qui a été positionné dans un nouveau repère.
	 *
	 * @param positionCentre La position du repère.
	 * @param rotation L'orientation du repère.
	 */
	public Point positionnementDansLeRepere( double rotation, Point positionCentre ){
		Point point = new Point( this );
		point.positionnerDansLeRepere( rotation, positionCentre );
		return point;
	}
}
