package geometrie;
import java.util.*;
/**
 * 
 * PolygoneConvexe est la classe qui implémente les polygones Convexes.
 *
 * Un polygone convexe est un polygone tel que chaque points du polygône 
 * est un baricentre à coefficient positif des sommets du polygones.
 */ 
public class PolygoneConvexe extends Polygone {
	/**
	 * Constructeur par recopie.
	 *
	 * L'utilisateur doit toujours utiliser ce constructeur avec un polygone dont 
	 * la frontière est la frontière est une enveloppe convexe.
	 * Les tests et vérifiactions de ces conditions sont à la charge de
	 * l'utilisateur de la classe.
	 * Une méthode statique est disponible à cet effet dans la classe 
	 * PolygoneConvexei : PolygoneConvexe#isConvexe()
     *
	 * @param polygone Le polygone de frontière convexe qui doit etre recopier.
	 */
	public PolygoneConvexe( Polygone polygone ){
		this( polygone.sommets );
	}

	/**
	 * Constructeur de la classe.
	 *
	 * L'utilisateur doit toujours utiliser ce constructeur avec une liste de
	 * sommets formant la frontière d'un polygone convexe.
	 * Les tests et vérifiactions de ces conditions sont à la charge de
	 * l'utilisateur de la classe.
	 * Une méthode statique est disponible à cet effet dans la classe 
	 * PolygoneConvexei : PolygoneConvexe#isConvexe()
     *
	 * @param sommets La liste de sommets formant la frontière d'un polygone 
     *                convexe
	 */
	public PolygoneConvexe( List<Point> sommets ){
		super( sommets );
	}

	/**
	 * Vérifie si le polygone est convexe.
	 * 
	 * @param polygone Le polygone à tester. 
     * @return Vraie si le polygone est convexe,
	 *         Faux sinon.
	 */
	public static boolean isConvexe( Polygone polygone ){
		return isConvexe( polygone.sommets );
	}

	/**
	 * Vérifie si une liste de points forme la frontière d'un polygone convexe.
	 *
	 * La liste de sommets ne doit pas comporter de doublon et la ligne brisée
	 * formé par ces sommets ne doit pas se couper elle même.
	 *  
	 * @param sommets La liste de sommets à tester. 
     * @return Vraie si le polygone est convexe,
	 *         Faux sinon.
	 */
	public static boolean isConvexe( List<Point> sommets ){
		Iterator it = sommets.iterator();
		Point p1 = (Point) ((LinkedList<Point>) sommets).getLast();
		Point p2 = (Point) it.next();
		Vecteur old = new Vecteur(p1,p2);
		p1 = p2;
		p2 = (Point) it.next();
		Vecteur arete = new Vecteur(p1,p2);
		boolean sensCourbure = ( ( old.produitVectoriel( arete ) ) >= 0.0 );
		while( it.hasNext() ){
			p1 = p2;
			p2 = (Point) it.next();
			old = arete;
			arete = new Vecteur(p1,p2);
			if(
				( ( old.produitVectoriel( arete ) ) >= 0.0 ) != sensCourbure
			) return false;
		}
		p1 = p2;
		p2 = ( (LinkedList<Point>) sommets).getFirst();
		old = arete;
		arete = new Vecteur(p1,p2);
		if(
			( ( old.produitVectoriel( arete ) ) >= 0.0 ) != sensCourbure
		) return false;
		return true;
	}

	/**
     * Teste si un point est à l'interieur de la figure.
	 *
	 * @param point Le point à tester. 
	 * @return Vraie si le point est à l'intérieur de la figure.
	 *         Faux sinon.
	 */
	public boolean isInside( Point point ){
		Iterator it = this.sommets.iterator();
		Point p1 = (Point) it.next();
		Point p2 = (Point) it.next();
		Vecteur arete = new Vecteur(p1,p2);
		boolean coteDeLArete = ( ( arete.produitVectoriel( new Vecteur(p1,point) ) ) >= 0.0 );
		while( it.hasNext() ){
			p1 = p2;
			p2 = (Point) it.next();
			arete = new Vecteur(p1,p2);
			if(
				( ( arete.produitVectoriel( new Vecteur(p1,point) ) ) >= 0.0 ) != coteDeLArete
			) return false;
		}
		p1 = p2;
		p2 = ( (LinkedList<Point>) this.sommets).getFirst();
		arete = new Vecteur(p1,p2);
		if(
			( ( arete.produitVectoriel( new Vecteur(p1,point) ) ) >= 0.0 ) != coteDeLArete
		) return false;
		return true;
	}
}
