package geometrie;
/**
 * Vecteur est la classe qui implémente les vecteurs mathémartiques.
 *
 * Elle fournit de nombreuses méthodes pour effectuer du calcul vectoriel.
 * On y trouve par exemple le calcul concernant :
 * <ul>
 * <li>la norme</li>
 * <li>le produit scalaire</li>
 * <li>le produit vectoriel</li>
 * <li>les rotations</li>
 * <li>les angles entre vecteurs</li>
 * </ul>
 * 
 * Cette classe implémente la notion mathématique de vecteurs appartenant
 * auix espaces vectoriels de dimention 2.
 */
public class Vecteur extends Couple {
	/**
	 * Constructeur de la classe.
	 *
	 * Il construit un vecteur à partir des coordonnées verticales et
	 * horizontale du vecteur
	 *
	 * @param x coordonnée horizontale du vecteur (abcisses)
	 * @param y coordonnée verticale du vecteur (ordonnée)
	 */
	public Vecteur(double x, double y){
		super(x,y);
	}

	/**
	 * Constructeur par recopie.
	 *
	 * @param v Le vecteur à recopier.
	 */
	public Vecteur( Vecteur v ){
		super(v);
	}

	/**
	 * Constructeur de la classe.
	 *
	 * Il construit un vecteur à partir d'un couple représentant les coordonnées
     * horizontales et verticales du vecteur
	 *
	 * @param c couple (x,y) représentnat les coordonnée horizontale x et 
	 *          verticales y du vecteur.
	 */
	public Vecteur( Couple c ){
		super(c);
	}

	/**
	 * Constructeur de la classe.
	 *
	 * Il construit un vecteur à partir de deux points du plan: l'origine du 
	 * vecteur et la fin du vecteur.
	 *
	 * @param origine L'origine du vecteur qui est un point du plan. 
	 * @param fin La fin du vecteur qui est un point du plan.
	 */
	public Vecteur( Point origine, Point fin){
		super( fin.getX() - origine.getX(), fin.getY() - origine.getY() );
	}


	/**
	 * Calcul le produit scalaire entre l'objet et un vecteur.
	 *
	 * Le produit scalaire entre deux vecteur mathématiques est un reel.
	 * Il permet d'évaluer si des vecteurs son colinéaires (parallèles)
	 * <p>
	 * Le produit scalaire pemret d'avaluer la quantité:
	 * (v1 | v2) = |v1| * |v2| cos( angle(v1,v2) )
	 *
	 * @param v le vecteur avec lequel on réalise le produit scalaire  this | v .
	 * @return Le réel codant la produit scalaire (this | v).
	 */
	public double produitScalaire( Vecteur v ){
		return v.x * this.x + v.y * this.y;
	}

	/**
	 * Calcul Le produit vectoriel entre l'objet et un vecteur.
	 *
	 * Le produit vectoriel entre deux vecteur mathématiques est un réel.
	 * Il permet d'évaluer si des vecteurs son perpendiculaires.
	 * <p>
	 * Le produit vectoriel pemret d'avaluer la quantité:
	 * v1 v v2  = |v1| * |v2| sin( angle(v1,v2) )
	 *
	 * @param v le vecteur avec lequel on réalise le produit vectoriel  this V v.
	 * @return Le réel codant le produit vectoriel this V v.
	 */
	public double produitVectoriel( Vecteur v ){
		return this.x * v.y - this.y * v.x;
	}

	/**
	 * Renvoie la norme au carré de l'objet
	 *
	 * @return Le réel codant la norme au carré de l'objet
	 */
	public double norme2(){
		return this.x * this.x + this.y * this.y;
	}

	/**
	 * Renvoie la norme de l'objet
	 *
	 * @return Le réel codant la norme de l'objet
	 */
	public double norme(){
		return java.lang.Math.sqrt( this.norme2() ); 
	}

	/**
	 * Calcul l'angle entre l'objet et un vecteur dans le sens direct.
	 *
	 * L'angle est en radian et est comprit entre -Pi et Pi.
	 * @param v vecteur avec lequel on clacul l'angle entre this et v.
	 * @return L'ange entre this et v, dans le sens direct, en radian et comprit 
	 *         entre [-Pi et Pi]
	 */
	public double angle( Vecteur v){
		double cos = this.produitScalaire(v)/( this.norme() * v.norme() );
		double sin = this.produitVectoriel(v)/( this.norme() * v.norme() );
		double angle = java.lang.Math.acos( cos );
		if( sin < 0 ){
			angle = - angle;
		}
		return angle;
	}

	/**
	 * Renvoie un nouveau vecteur qui est une rotation de l'objet.
	 *
	 * La rotation se fait dans le sens direct.
	 *
	 * @param angle angle de la rotation en radian.
	 * @return Vecteur qui a été pivoté.
	 */
	public Vecteur rotation( double angle ){
		Vecteur resultat = new Vecteur( this );
		resultat.effectuerRotation( angle );
		return resultat;
	}

	/**
	 * Modifie l'objet en effectuant une rotation.
	 *
	 * La rotation se fait dans le sens direct.
	 *
	 * @param angle angle de la rotation en radian.
	 */
	public void effectuerRotation( double angle ){
		double tmp = this.x;
		this.x = java.lang.Math.cos(angle) * this.x 
				 - java.lang.Math.sin(angle) * this.y;
		this.y = java.lang.Math.sin(angle) * tmp
				 + java.lang.Math.cos(angle) * this.y;
	}

	/**
	 * Renvoie le vecteur obtenu en additionnant l'objet avec un vecteur passé en parametre.
	 *
	 * @param v Un vecteur
	 * @return l'addition de l'objet avec le vecteur passé en paramètre.
	 */
	public Vecteur plus( Vecteur v ){
		return new Vecteur( this.x + v.x, this.y + v.y );
	}
}
